@@ -11,11 +11,15 @@
 #define GREEN "\x1b[32;06m"
 #define YELLOW "\x1b[33;06m"
 
-int debug_level = -1;
+int debug_level = 0;
+
+void debug_set_level(DebugLevel dl)
+{
+	debug_level = (dl<0)?0:((dl > DEBUG_ERROR)?DEBUG_ERROR:dl);
+}
 
 
-
-void debug_printf(int dp,char *format, ...)
+void debug_printf_real(DebugLevel dp, const char *file,const int line,const char *function, const char *format,...)
 {
 	if(debug_level >= dp)
 	{
@@ -23,24 +27,22 @@ void debug_printf(int dp,char *format, ...)
 		va_start(arglist,format);
 		if(dp == DEBUG_INFO)
 		{
-			printf(GREEN"INFO:\t"RESET);
+			printf(GREEN"INFO:"RESET"    %s %s():#%d:\t",file,function,line);
 		}
 		else if(dp == DEBUG_WARNING)
 		{
-			printf(YELLOW"WARNING:\t"RESET);
+			printf(YELLOW"WARNING:"RESET" %s %s():#%i:\t",file,function,line);
 		}
 		else
 		{
-			printf(DARKRED"ERROR:\t"RESET);
+			printf(DARKRED"ERROR:"RESET"   %s %s():#%i:\t",file,function,line);
 		}
 		vprintf(format, arglist);
 		if(format[strlen(format)-1] != '\n')
 		{
 			printf("\n");
 		}
+		fflush(NULL);
 		va_end(arglist);
 	}
 }
-
-
-
