@@ -1,22 +1,22 @@
 /* libmpdclient
    (c)2003-2004 by Warren Dukes (shank@mercury.chem.pitt.edu)
    This project's homepage is: http://www.musicpd.org
-  
+
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
-                                                                                
+
    - Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
-                                                                                
+
    - Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
-                                                                                
+
    - Neither the name of the Music Player Daemon nor the names of its
    contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.
-                                                                                
+
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -36,21 +36,29 @@
 
 #include <errno.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
 #include <stdio.h>
 #include <sys/param.h>
 
 
 #include <string.h>
 #include <strings.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <stdarg.h>
 
+
+#ifdef WIN32
+#include <ws2tcpip.h>
+#include <winsock.h>
+#else
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#endif
+
+
 #ifndef MPD_NO_IPV6
 #ifdef AF_INET6
 #define MPD_HAVE_IPV6
@@ -81,12 +89,12 @@ char * mpdTagItemKeys[MPD_TAG_NUM_OF_ITEM_TYPES] =
 
 
 #ifdef MPD_HAVE_IPV6        
-int mpd_ipv6Supported() {
-        int s;          
-        s = socket(AF_INET6,SOCK_STREAM,0);
-        if(s == -1) return 0;
-        close(s);       
-        return 1;                       
+static int mpd_ipv6Supported() {
+	int s;          
+	s = socket(AF_INET6,SOCK_STREAM,0);
+	if(s == -1) return 0;
+	close(s);       
+	return 1;                       
 }                       
 #endif  
 
@@ -95,17 +103,17 @@ static char * mpd_sanitizeArg(const char * arg) {
 	char * ret;
 	register const char *c;
 	register char *rc;
-	
+
 	/* 
-	unsigned int count = 0;
-	
-	c = arg;
-	for(i = strlen(arg); i != 0; --i) {
-		if(*c=='"' || *c=='\\') count++;
-		c++;
-	}
-	ret = malloc(strlen(arg)+count+1);
-	*/
+	   unsigned int count = 0;
+
+	   c = arg;
+	   for(i = strlen(arg); i != 0; --i) {
+	   if(*c=='"' || *c=='\\') count++;
+	   c++;
+	   }
+	   ret = malloc(strlen(arg)+count+1);
+	   */
 	/* instead of counting in that loop above, just
 	 * use a bit more memory and half running time
 	 */
@@ -122,26 +130,70 @@ static char * mpd_sanitizeArg(const char * arg) {
 	return ret;
 }
 
-mpd_ReturnElement * mpd_newReturnElement(const char * name, const char * value)
+/* DONT USE THIS FUNCTION UNLESS YOU'RE ABSOLUTELY SURE WHAT YOU'RE DOING !!!! */
+static void mpd_setReturnElement(mpd_Connection * connection, const char * name, const char * value)
 {
-	mpd_ReturnElement * ret = malloc(sizeof(mpd_ReturnElement));
-
-	ret->name = (char *)strdup(name);
-	ret->value = (char *)strdup(value);
-
-	return ret;
+	if (!connection) return;
+	if (!connection->returnElement) {
+		connection->returnElement = malloc(sizeof(mpd_ReturnElement));
+		connection->returnElement->name = malloc(strlen(value)+strlen(name)+3);
+	} else {
+		if ((strlen(name)+strlen(value)) > (strlen(connection->returnElement->name)+strlen(connection->returnElement->value))) {
+			free(connection->returnElement->name);
+			connection->returnElement->name = malloc(strlen(value)+strlen(name)+3);
+		}
+	}
+	memcpy(connection->returnElement->name, name, strlen(name)+strlen(value)+3);
+	connection->returnElement->value = connection->returnElement->name + strlen(name) + 2;
 }
 
-void mpd_freeReturnElement(mpd_ReturnElement * re) {
-	free(re->name);
-	free(re->value);
-	free(re);
+static void mpd_freeReturnElement(mpd_Connection * connection) {
+	if (connection->returnElement) {
+		free(connection->returnElement->name);
+		free(connection->returnElement);
+		connection->returnElement = NULL;
+	}
 }
 
 void mpd_setConnectionTimeout(mpd_Connection * connection, float timeout) {
-		connection->timeout.tv_sec = (int)timeout;
-		connection->timeout.tv_usec = (int)(timeout*1e6 -
-				connection->timeout.tv_sec*1000000+0.5);
+	connection->timeout.tv_sec = (int)timeout;
+	connection->timeout.tv_usec = (int)(timeout*1e6 -
+			connection->timeout.tv_sec*1000000+0.5);
+}
+
+static int mpd_parseWelcome(mpd_Connection * connection, const char * host, int port,
+		char * rt, char * output) {
+
+	char * tmp;
+	char * test;
+	int i;
+
+	if(strncmp(output,MPD_WELCOME_MESSAGE,strlen(MPD_WELCOME_MESSAGE))) {
+		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
+				"mpd not running on port %i on host \"%s\"",
+				port,host);
+		connection->error = MPD_ERROR_NOTMPD;
+		return 1;
+	}
+
+	tmp = &output[strlen(MPD_WELCOME_MESSAGE)];
+
+	for(i=0;i<3;i++) {
+		if(tmp) connection->version[i] = strtol(tmp,&test,10);
+
+		if (!tmp || (test[0] != '.' && test[0] != '\0')) {
+			snprintf(connection->errorStr,
+					MPD_BUFFER_MAX_LENGTH,
+					"error parsing version number at "
+					"\"%s\"",
+					&output[strlen(MPD_WELCOME_MESSAGE)]);
+			connection->error = MPD_ERROR_NOTMPD;
+			return 1;
+		}
+		tmp = ++test;
+	}
+
+	return 0;
 }
 
 mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
@@ -172,6 +224,17 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	connection->listOks = 0;
 	connection->doneListOk = 0;
 	connection->returnElement = NULL;
+	connection->request = NULL;
+#ifdef WIN32
+	WSADATA wsaData;
+	if ((WSAStartup(MAKEWORD(2, 2), &wsaData)) != 0 ||
+			LOBYTE(wsaData.wVersion) != 2 || HIBYTE(wsaData.wVersion) != 2 ) {
+		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
+				"Could not find usable WinSock DLL.");
+		connection->error = MPD_ERROR_SYSTEM;
+		return connection;
+	}
+#endif
 
 	if(!(he=gethostbyname(host))) {
 		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
@@ -190,34 +253,34 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	sin6.sin6_port = htons(port);
 #endif
 	switch(he->h_addrtype) {
-	case AF_INET:
-		memcpy((char *)&sin.sin_addr.s_addr,(char *)he->h_addr,
-				he->h_length);
-		dest = (struct sockaddr *)&sin;
-		destlen = sizeof(struct sockaddr_in);
-		break;
+		case AF_INET:
+			memcpy((char *)&sin.sin_addr.s_addr,(char *)he->h_addr,
+					he->h_length);
+			dest = (struct sockaddr *)&sin;
+			destlen = sizeof(struct sockaddr_in);
+			break;
 #ifdef MPD_HAVE_IPV6
-	case AF_INET6:
-		if(!mpd_ipv6Supported()) {
-			strcpy(connection->errorStr,"no IPv6 suuport but a "
-					"IPv6 address found\n");
+		case AF_INET6:
+			if(!mpd_ipv6Supported()) {
+				strcpy(connection->errorStr,"no IPv6 suuport but a "
+						"IPv6 address found\n");
+				connection->error = MPD_ERROR_SYSTEM;
+				return connection;
+			}
+			memcpy((char *)&sin6.sin6_addr.s6_addr,(char *)he->h_addr,
+					he->h_length);
+			dest = (struct sockaddr *)&sin6;
+			destlen = sizeof(struct sockaddr_in6);
+			break;
+#endif
+		default:
+			strcpy(connection->errorStr,"address type is not IPv4 or "
+					"IPv6\n");
 			connection->error = MPD_ERROR_SYSTEM;
 			return connection;
-		}
-		memcpy((char *)&sin6.sin6_addr.s6_addr,(char *)he->h_addr,
-				he->h_length);
-		dest = (struct sockaddr *)&sin6;
-		destlen = sizeof(struct sockaddr_in6);
-		break;
-#endif
-	default:
-		strcpy(connection->errorStr,"address type is not IPv4 or "
-				"IPv6\n");
-		connection->error = MPD_ERROR_SYSTEM;
-		return connection;
-		break;
+			break;
 	}
-	
+
 	if((connection->sock = socket(dest->sa_family,SOCK_STREAM,0))<0) {
 		strcpy(connection->errorStr,"problems creating socket");
 		connection->error = MPD_ERROR_SYSTEM;
@@ -228,15 +291,22 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 
 	/* connect stuff */
 	{
-		int flags = fcntl(connection->sock, F_GETFL, 0);
+#ifdef WIN32
+		int iMode = 1; // 0 = blocking, else non-blocking
+		ioctlsocket(connection->sock, FIONBIO, (u_long FAR*) &iMode);
+		if(connect(connection->sock,dest,destlen) == SOCKET_ERROR
+				&& WSAGetLastError() != WSAEWOULDBLOCK)
+#else
+			int flags = fcntl(connection->sock, F_GETFL, 0);
 		fcntl(connection->sock, F_SETFL, flags | O_NONBLOCK);
 
 		if(connect(connection->sock,dest,destlen)<0 && 
 				errno!=EINPROGRESS) 
+#endif
 		{
 			snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
 					"problems connecting to \"%s\" on port"
-				 	" %i",host,port);
+					" %i",host,port);
 			connection->error = MPD_ERROR_CONNPORT;
 			return connection;
 		}
@@ -250,38 +320,36 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 		if((err = select(connection->sock+1,&fds,NULL,NULL,&tv)) == 1) {
 			int readed;
 			readed = recv(connection->sock,
-				&(connection->buffer[connection->buflen]),
-				MPD_BUFFER_MAX_LENGTH-connection->buflen,0);
+					&(connection->buffer[connection->buflen]),
+					MPD_BUFFER_MAX_LENGTH-connection->buflen,0);
 			if(readed<=0) {
 				snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
-					"problems getting a response from"
-					" \"%s\" on port %i : %s",host,
-					port, strerror(errno));
+						"problems getting a response from"
+						" \"%s\" on port %i : %s",host,
+						port, strerror(errno));
 				connection->error = MPD_ERROR_NORESPONSE;
 				return connection;
 			}
 			connection->buflen+=readed;
 			connection->buffer[connection->buflen] = '\0';
-			tv.tv_sec = connection->timeout.tv_sec;
-			tv.tv_usec = connection->timeout.tv_usec;
 		}
 		else if(err<0) {
 			switch(errno) {
-			case EINTR:
-				continue;
-			default:
-				snprintf(connection->errorStr,
-					MPD_BUFFER_MAX_LENGTH,
-					"problems connecting to \"%s\" on port"
-				 	" %i",host,port);
-				connection->error = MPD_ERROR_CONNPORT;
-				return connection;
+				case EINTR:
+					continue;
+				default:
+					snprintf(connection->errorStr,
+							MPD_BUFFER_MAX_LENGTH,
+							"problems connecting to \"%s\" on port"
+							" %i",host,port);
+					connection->error = MPD_ERROR_CONNPORT;
+					return connection;
 			}
 		}
 		else {
 			snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
-				"timeout in attempting to get a response from"
-				 " \"%s\" on port %i",host,port);
+					"timeout in attempting to get a response from"
+					" \"%s\" on port %i",host,port);
 			connection->error = MPD_ERROR_NORESPONSE;
 			return connection;
 		}
@@ -292,55 +360,10 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	strcpy(connection->buffer,rt+1);
 	connection->buflen = strlen(connection->buffer);
 
-	if(strncmp(output,MPD_WELCOME_MESSAGE,strlen(MPD_WELCOME_MESSAGE))) {
-		free(output);
-		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
-				"mpd not running on port %i on host \"%s\"",
-				port,host);
-		connection->error = MPD_ERROR_NOTMPD;
-		return connection;
-	}
-
-	{
-		char * test;
-		char * version[3];
-		char * tmp = &output[strlen(MPD_WELCOME_MESSAGE)];
-		char * search = ".";
-		int i;
-
-		for(i=0;i<3;i++) {
-			char * tok;
-			if(i==3) search = " ";
-			version[i] = strtok_r(tmp,search,&tok);
-			if(!version[i]) {
-				free(output);
-				snprintf(connection->errorStr,
-					MPD_BUFFER_MAX_LENGTH,
-					"error parsing version number at "
-					"\"%s\"",
-					&output[strlen(MPD_WELCOME_MESSAGE)]);
-				connection->error = MPD_ERROR_NOTMPD;
-				return connection;
-			}
-			connection->version[i] = strtol(version[i],&test,10);
-			if(version[i]==test || *test!='\0') {
-				free(output);
-				snprintf(connection->errorStr,
-					MPD_BUFFER_MAX_LENGTH,
-					"error parsing version number at "
-					"\"%s\"",
-					&output[strlen(MPD_WELCOME_MESSAGE)]);
-				connection->error = MPD_ERROR_NOTMPD;
-				return connection;
-			}
-			tmp = NULL;
-		}
-	}
+	if(mpd_parseWelcome(connection,host,port,rt,output) == 0) connection->doneProcessing = 1;
 
 	free(output);
 
-	connection->doneProcessing = 1;
-
 	return connection;
 }
 
@@ -350,12 +373,20 @@ void mpd_clearError(mpd_Connection * connection) {
 }
 
 void mpd_closeConnection(mpd_Connection * connection) {
+#ifdef WIN32
+	closesocket(connection->sock);
+#else
 	close(connection->sock);
+#endif
 	if(connection->returnElement) free(connection->returnElement);
+	if(connection->request) free(connection->request);
 	free(connection);
+#ifdef WIN32
+	WSACleanup();
+#endif
 }
 
-void mpd_executeCommand(mpd_Connection * connection, char * command) {
+static void mpd_executeCommand(mpd_Connection * connection, char * command) {
 	int ret;
 	struct timeval tv;
 	fd_set fds;
@@ -379,16 +410,15 @@ void mpd_executeCommand(mpd_Connection * connection, char * command) {
 			(ret==-1 && errno==EINTR)) {
 		ret = send(connection->sock,commandPtr,commandLen,
 #ifdef WIN32
-			   ioctlsocket(connection->sock, commandLen, commandPtr));
-#endif
-#ifndef WIN32
-				MSG_DONTWAIT);
+				0);
+#else
+		MSG_DONTWAIT);
 #endif
 		if(ret<=0)
 		{
 			if(ret==EAGAIN || ret==EINTR) continue;
 			snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
-				"problems giving command \"%s\"",command);
+					"problems giving command \"%s\"",command);
 			connection->error = MPD_ERROR_SENDING;
 			return;
 		}
@@ -403,7 +433,7 @@ void mpd_executeCommand(mpd_Connection * connection, char * command) {
 	if(commandLen>0) {
 		perror("");
 		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
-			"timeout sending command \"%s\"",command);
+				"timeout sending command \"%s\"",command);
 		connection->error = MPD_ERROR_TIMEOUT;
 		return;
 	}
@@ -414,7 +444,7 @@ void mpd_executeCommand(mpd_Connection * connection, char * command) {
 	}
 }
 
-void mpd_getNextReturnElement(mpd_Connection * connection) {
+static void mpd_getNextReturnElement(mpd_Connection * connection) {
 	char * output = NULL;
 	char * rt = NULL;
 	char * name = NULL;
@@ -425,15 +455,15 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 	int readed;
 	char * bufferCheck = NULL;
 	int err;
+	int pos;
 
-	if(connection->returnElement) mpd_freeReturnElement(connection->returnElement);
-	connection->returnElement = NULL;
 
 	if(connection->doneProcessing || (connection->listOks &&
-			connection->doneListOk)) 
+				connection->doneListOk)) 
 	{
 		strcpy(connection->errorStr,"already done processing current command");
 		connection->error = 1;
+		mpd_freeReturnElement(connection);
 		return;
 	}
 
@@ -454,6 +484,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 			connection->error = MPD_ERROR_BUFFEROVERRUN;
 			connection->doneProcessing = 1;
 			connection->doneListOk = 0;
+			mpd_freeReturnElement(connection);
 			return;
 		}
 		bufferCheck = connection->buffer+connection->buflen;
@@ -463,25 +494,23 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 		FD_SET(connection->sock,&fds);
 		if((err = select(connection->sock+1,&fds,NULL,NULL,&tv) == 1)) {
 			readed = recv(connection->sock,
-				connection->buffer+connection->buflen,
-				MPD_BUFFER_MAX_LENGTH-connection->buflen,
+					connection->buffer+connection->buflen,
+					MPD_BUFFER_MAX_LENGTH-connection->buflen,
 #ifdef WIN32
-				ioctlsocket(connection->sock, 
-					    commandLen, 
-					    commandPtr));
-#endif
-#ifndef WIN32
-				MSG_DONTWAIT);
+					0);
+#else
+			MSG_DONTWAIT);
 #endif
 			if(readed<0 && (errno==EAGAIN || errno==EINTR)) {
 				continue;
 			}
 			if(readed<=0) {
 				strcpy(connection->errorStr,"connection"
-					" closed");
+						" closed");
 				connection->error = MPD_ERROR_CONNCLOSED;
 				connection->doneProcessing = 1;
 				connection->doneListOk = 0;
+				mpd_freeReturnElement(connection);
 				return;
 			}
 			connection->buflen+=readed;
@@ -493,6 +522,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 			connection->error = MPD_ERROR_TIMEOUT;
 			connection->doneProcessing = 1;
 			connection->doneListOk = 0;
+			mpd_freeReturnElement(connection);
 			return;
 		}
 	}
@@ -509,6 +539,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 		connection->listOks = 0;
 		connection->doneProcessing = 1;
 		connection->doneListOk = 0;
+		mpd_freeReturnElement(connection);
 		return;
 	}
 
@@ -522,6 +553,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 			connection->doneListOk = 1;
 			connection->listOks--;
 		}
+		mpd_freeReturnElement(connection);
 		return;
 	}
 
@@ -529,7 +561,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 		char * test;
 		char * needle;
 		int val;
-	
+
 		strcpy(connection->errorStr, output);
 		connection->error = MPD_ERROR_ACK;
 		connection->errorCode = MPD_ACK_ERROR_UNK;
@@ -545,15 +577,24 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 		val = strtol(test+1, &test, 10);
 		if(*test != ']') return;
 		connection->errorAt = val;
+		mpd_freeReturnElement(connection);
 		return;
 	}
 
-	name = strtok_r(output,":",&tok);
-	if(name && (value = strtok_r(NULL,"",&tok)) && value[0]==' ') {
-		connection->returnElement = mpd_newReturnElement(name,&(value[1]));
+	tok = strchr(output, ':');
+	if (!tok) return;
+	pos = tok - output;
+	tok[0]='\0';
+	value = tok+1;
+	//name = (char *)malloc(pos+1);
+	//strncpy(name, output, pos);
+	//name[pos] = '\0';
+	if(value && value[0]==' ') {
+		mpd_setReturnElement(connection, output, &(value[1]));
 	}
 	else {
-		if(!name || !value) {
+		if(!value) {
+			tok[0]=':';
 			snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
 					"error parsing: %s",output);
 		}
@@ -563,6 +604,7 @@ void mpd_getNextReturnElement(mpd_Connection * connection) {
 		}
 		connection->errorStr[MPD_BUFFER_MAX_LENGTH] = '\0';
 		connection->error = 1;
+		mpd_freeReturnElement(connection);
 	}
 }
 
@@ -573,7 +615,7 @@ void mpd_finishCommand(mpd_Connection * connection) {
 	}
 }
 
-void mpd_finishListOkCommand(mpd_Connection * connection) {
+static void mpd_finishListOkCommand(mpd_Connection * connection) {
 	while(!connection->doneProcessing && connection->listOks && 
 			!connection->doneListOk ) 
 	{
@@ -595,12 +637,8 @@ void mpd_sendStatusCommand(mpd_Connection * connection) {
 mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 	mpd_Status * status;
 
-	/*mpd_executeCommand(connection,"status\n");
-		
-	if(connection->error) return NULL;*/
-
 	if(connection->doneProcessing || (connection->listOks && 
-			connection->doneListOk))
+				connection->doneListOk))
 	{
 		return NULL;
 	}
@@ -615,6 +653,7 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 	status->playlistLength = -1;
 	status->state = -1;
 	status->song = 0;
+	status->songid = 0;
 	status->elapsedTime = 0;
 	status->totalTime = 0;
 	status->bitRate = 0;
@@ -670,16 +709,13 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 			status->songid = atoi(re->value);
 		}
 		else if(strcmp(re->name,"time")==0) {
-			char * tok;
 			char * copy;
 			char * temp;
 			copy = strdup(re->value);
-			temp = strtok_r(copy,":",&tok);
-			if(temp) {
-				status->elapsedTime = atoi(temp);
-				temp = strtok_r(NULL,"",&tok);
-				if(temp) status->totalTime = atoi(temp);
-			}
+			status->elapsedTime = atoi(copy);
+			temp = strchr(copy, ':');
+			if (temp)
+				status->totalTime = atoi(++temp);
 			free(copy);
 		}
 		else if(strcmp(re->name,"error")==0) {
@@ -692,18 +728,16 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 			status->updatingDb = atoi(re->value);
 		}
 		else if(strcmp(re->name,"audio")==0) {
-			char * tok;
 			char * copy;
 			char * temp;
 			copy = strdup(re->value);
-			temp = strtok_r(copy,":",&tok);
+			status->sampleRate = atoi(copy);
+			temp = strchr(copy,':');
 			if(temp) {
-				status->sampleRate = atoi(temp);
-				temp = strtok_r(NULL,":",&tok);
+				status->bits = atoi(++temp);
+				temp = strchr(temp,':');
 				if(temp) {
-					status->bits = atoi(temp);
-					temp = strtok_r(NULL,"",&tok);
-					if(temp) status->channels = atoi(temp);
+					status->channels = atoi(++temp);
 				}
 			}
 			free(copy);
@@ -743,11 +777,11 @@ mpd_Stats * mpd_getStats(mpd_Connection * connection) {
 	mpd_Stats * stats;
 
 	/*mpd_executeCommand(connection,"stats\n");
-		
-	if(connection->error) return NULL;*/
+
+	  if(connection->error) return NULL;*/
 
 	if(connection->doneProcessing || (connection->listOks && 
-			connection->doneListOk))
+				connection->doneListOk))
 	{
 		return NULL;
 	}
@@ -810,7 +844,7 @@ void mpd_freeStats(mpd_Stats * stats) {
 	free(stats);
 }
 
-void mpd_initSong(mpd_Song * song) {
+static void mpd_initSong(mpd_Song * song) {
 	song->file = NULL;
 	song->artist = NULL;
 	song->album = NULL;
@@ -827,7 +861,7 @@ void mpd_initSong(mpd_Song * song) {
 	song->id = MPD_SONG_NO_ID;
 }
 
-void mpd_finishSong(mpd_Song * song) {
+static void mpd_finishSong(mpd_Song * song) {
 	if(song->file) free(song->file);
 	if(song->artist) free(song->artist);
 	if(song->album) free(song->album);
@@ -871,11 +905,11 @@ mpd_Song * mpd_songDup(mpd_Song * song) {
 	return ret;
 }
 
-void mpd_initDirectory(mpd_Directory * directory) {
+static void mpd_initDirectory(mpd_Directory * directory) {
 	directory->path = NULL;
 }
 
-void mpd_finishDirectory(mpd_Directory * directory) {
+static void mpd_finishDirectory(mpd_Directory * directory) {
 	if(directory->path) free(directory->path);
 }
 
@@ -883,7 +917,7 @@ mpd_Directory * mpd_newDirectory () {
 	mpd_Directory * directory = malloc(sizeof(mpd_Directory));;
 
 	mpd_initDirectory(directory);
-	
+
 	return directory;
 }
 
@@ -901,11 +935,11 @@ mpd_Directory * mpd_directoryDup(mpd_Directory * directory) {
 	return ret;
 }
 
-void mpd_initPlaylistFile(mpd_PlaylistFile * playlist) {
+static void mpd_initPlaylistFile(mpd_PlaylistFile * playlist) {
 	playlist->path = NULL;
 }
 
-void mpd_finishPlaylistFile(mpd_PlaylistFile * playlist) {
+static void mpd_finishPlaylistFile(mpd_PlaylistFile * playlist) {
 	if(playlist->path) free(playlist->path);
 }
 
@@ -930,11 +964,11 @@ mpd_PlaylistFile * mpd_playlistFileDup(mpd_PlaylistFile * playlist) {
 	return ret;
 }
 
-void mpd_initInfoEntity(mpd_InfoEntity * entity) {
+static void mpd_initInfoEntity(mpd_InfoEntity * entity) {
 	entity->info.directory = NULL;
 } 
 
-void mpd_finishInfoEntity(mpd_InfoEntity * entity) {
+static void mpd_finishInfoEntity(mpd_InfoEntity * entity) {
 	if(entity->info.directory) {
 		if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
 			mpd_freeDirectory(entity->info.directory);
@@ -950,7 +984,7 @@ void mpd_finishInfoEntity(mpd_InfoEntity * entity) {
 
 mpd_InfoEntity * mpd_newInfoEntity() {
 	mpd_InfoEntity * entity = malloc(sizeof(mpd_InfoEntity));
-	
+
 	mpd_initInfoEntity(entity);
 
 	return entity;
@@ -961,7 +995,7 @@ void mpd_freeInfoEntity(mpd_InfoEntity * entity) {
 	free(entity);
 }
 
-void mpd_sendInfoCommand(mpd_Connection * connection, char * command) {
+static void mpd_sendInfoCommand(mpd_Connection * connection, char * command) {
 	mpd_executeCommand(connection,command);
 }
 
@@ -969,7 +1003,7 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 	mpd_InfoEntity * entity = NULL;
 
 	if(connection->doneProcessing || (connection->listOks && 
-			connection->doneListOk))
+				connection->doneListOk))
 	{
 		return NULL;
 	}
@@ -1061,7 +1095,7 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 					strcmp(re->name, "Composer") == 0) {
 				entity->info.song->composer = strdup(re->value);
 			}                                                    			
-			
+
 		}
 		else if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
 		}
@@ -1074,11 +1108,11 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 	return entity;
 }
 
-char * mpd_getNextReturnElementNamed(mpd_Connection * connection, 
+static char * mpd_getNextReturnElementNamed(mpd_Connection * connection, 
 		const char * name) 
 {
 	if(connection->doneProcessing || (connection->listOks && 
-			connection->doneListOk)) 
+				connection->doneListOk)) 
 	{
 		return NULL;
 	}
@@ -1094,13 +1128,13 @@ char * mpd_getNextReturnElementNamed(mpd_Connection * connection,
 	return NULL;
 }
 
-char * mpd_getNextTag(mpd_Connection * connection,int table) {
-	if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
-	{
-		return mpd_getNextReturnElementNamed(connection,mpdTagItemKeys[table]);
+	char * mpd_getNextTag(mpd_Connection * connection,int table) {
+		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
+		{
+			return mpd_getNextReturnElementNamed(connection,mpdTagItemKeys[table]);
+		}
+		return NULL;
 	}
-	return NULL;
-}
 char * mpd_getNextArtist(mpd_Connection * connection) {
 	return mpd_getNextReturnElementNamed(connection,"Artist");
 }
@@ -1182,89 +1216,6 @@ void mpd_sendSearchCommand(mpd_Connection * connection, int table,
 	free(string);
 	free(sanitStr);
 }
-void mpd_sendSearchTagCommand(mpd_Connection *connection, ...)
-{
-	va_list arglist;
-	va_start(arglist, connection);
-	mpd_sendVSearchTagCommand(connection, arglist);
-	va_end(arglist);
-}
-
-void mpd_sendVSearchTagCommand(mpd_Connection * connection, va_list arglist) 
-{
-	char *st, *str;
-	char * string=NULL;
-	int table;
-	char * sanitStr;
-	string = realloc(string,strlen("search")+1);
-	strcpy(string, "search");
-	while((table = va_arg(arglist, int)) != -1)
-	{
-		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
-		{
-			st = mpdTagItemKeys[table];
-			str = va_arg(arglist,char *);
-			sanitStr = mpd_sanitizeArg(str);
-			string = realloc(string, strlen(string)+strlen(st)+strlen(sanitStr)+6);
-			sprintf(string, "%s %s \"%s\"",string,st,sanitStr);
-			free(sanitStr);
-		}	
-		else {
-			connection->error = 1;
-			sprintf(connection->errorStr,"unknown table for search %i",table);
-			va_end(arglist);
-			return;
-		}
-	}
-	/* set the last to '\n', should be sufficient space in the string for this */
-	sprintf(string, "%s\n", string);
-	mpd_sendInfoCommand(connection,string);
-	free(string);
-}
-
-
-void mpd_sendFindTagCommand(mpd_Connection *connection, ...)
-{
-	va_list arglist;
-	va_start(arglist, connection);
-	mpd_sendVFindTagCommand(connection, arglist);
-	va_end(arglist);
-}
-
-
-
-
-void mpd_sendVFindTagCommand(mpd_Connection * connection, va_list arglist) 
-{
-	char *st, *str;
-	char * string=NULL;
-	int table;
-	char * sanitStr;
-	string = realloc(string,strlen("find")+1);
-	strcpy(string, "find");
-	while((table = va_arg(arglist, int)) != -1)
-	{
-		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
-		{
-			st = mpdTagItemKeys[table];
-			str = va_arg(arglist,char *);
-			sanitStr = mpd_sanitizeArg(str);
-			string = realloc(string, strlen(string)+strlen(st)+strlen(sanitStr)+6);
-			sprintf(string, "%s %s \"%s\"",string,st,sanitStr);
-			free(sanitStr);
-		}	
-		else {
-			connection->error = 1;
-			sprintf(connection->errorStr,"unknown table for find %i",table);
-			va_end(arglist);
-			return;
-		}
-	}
-	/* set the last to '\n', should be sufficient space in the string for this */
-	sprintf(string, "%s\n", string);
-	mpd_sendInfoCommand(connection,string);
-	free(string);
-}
 
 void mpd_sendFindCommand(mpd_Connection * connection, int table, 
 		const char * str) 
@@ -1313,52 +1264,6 @@ void mpd_sendListCommand(mpd_Connection * connection, int table,
 	free(string);
 }
 
-void mpd_sendListTagCommand(mpd_Connection *connection, int ret_table, ...)
-{
-	va_list arglist;
-	va_start(arglist, ret_table);
-	mpd_sendVListTagCommand(connection, ret_table, arglist);
-	va_end(arglist);
-}
-void mpd_sendVListTagCommand(mpd_Connection * connection,int ret_table, va_list arglist) 
-{
-	char *st, *str;
-	char * string=NULL;
-	int table;
-	char * sanitStr;
-	if(ret_table < 0 && ret_table >= MPD_TAG_NUM_OF_ITEM_TYPES)
-	{
-		connection->error = 1;
-		sprintf(connection->errorStr,"unknown ret_table for search %i",ret_table);
-		return;
-	}
-
-	string = realloc(string,strlen("list")+3+strlen(mpdTagItemKeys[ret_table]));
-	sprintf(string, "list %s",mpdTagItemKeys[ret_table]);
-	while((table = va_arg(arglist, int)) != -1)
-	{
-		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
-		{
-			st = mpdTagItemKeys[table];
-			str = va_arg(arglist,char *);
-			sanitStr = mpd_sanitizeArg(str);
-			string = realloc(string, strlen(string)+strlen(st)+strlen(sanitStr)+7);
-			sprintf(string, "%s %s \"%s\"",string,st,sanitStr);
-			free(sanitStr);
-		}	
-		else {
-			connection->error = 1;
-			sprintf(connection->errorStr,"unknown table for search %i",table);
-			va_end(arglist);
-			return;
-		}
-	}
-	/* set the last to '\n', should be sufficient space in the string for this */
-	sprintf(string,"%s\n", string);
-	mpd_sendInfoCommand(connection,string);
-	free(string);
-}
-
 
 void mpd_sendAddCommand(mpd_Connection * connection, const char * file) {
 	char * sFile = mpd_sanitizeArg(file);
@@ -1654,3 +1559,125 @@ void mpd_freeOutputElement(mpd_OutputEntity * output) {
 	free(output->name);
 	free(output);
 }
+
+
+
+/**
+ * mpd_sendNotCommandsCommand
+ * odd naming, but it gets the not allowed commands
+ */
+
+void mpd_sendNotCommandsCommand(mpd_Connection * connection) {
+	mpd_executeCommand(connection,"notcommands\n");
+}
+
+/**
+ * mpd_sendCommandsCommand
+ * odd naming, but it gets the allowed commands
+ */
+
+void mpd_sendCommandsCommand(mpd_Connection * connection) {
+	mpd_executeCommand(connection,"commands\n");
+}
+
+char * mpd_getNextCommand(mpd_Connection * connection) {
+	return mpd_getNextReturnElementNamed(connection,"command");
+}
+
+void mpd_startSearch(mpd_Connection * connection,int exact) {
+	if(connection->request) {
+		/* search/find allready in progress */
+		/* TODO: set error here?  */
+		return;
+	}
+	if(exact){
+		connection->request = strdup("find");
+	}
+	else{
+		connection->request = strdup("search");
+	}
+}
+
+
+void mpd_startFieldSearch(mpd_Connection * connection,int field) {
+	if(connection->request) {
+		/* search/find allready in progress */
+		/* TODO: set error here?  */
+		return;
+	}
+	if(field < 0 || field >= MPD_TAG_NUM_OF_ITEM_TYPES) {
+		/* set error here */
+		return;
+	}
+
+	connection->request = malloc(sizeof(char)*(
+				/* length of the field name */
+				strlen(mpdTagItemKeys[field])+ 	
+				/* "list"+space+\0 */
+				6 
+				));
+	sprintf(connection->request, "list %s", mpdTagItemKeys[field]);
+}
+
+
+
+void mpd_addConstraintSearch(mpd_Connection *connection, 
+		int field, 
+		char *name)
+{
+	char *arg = NULL;
+	if(!connection->request){
+		return;
+	}
+	if(name == NULL) {
+		return;
+	}
+	if(field < 0 || field >= MPD_TAG_NUM_OF_ITEM_TYPES) {
+		return;
+	}
+	/* clean up the query */
+	arg = mpd_sanitizeArg(name);
+	/* create space for the query */
+	connection->request = realloc(connection->request, (
+			 /* length of the old string */
+			 strlen(connection->request)+ 
+			 /* space between */
+			 1+
+			 /* length of the field name */
+			 strlen(mpdTagItemKeys[field])+ 
+			 /* space plus starting " */
+			 2+
+			 /* length of search term */
+			 strlen(arg)+
+			 /* closign " +\0 that is added sprintf */
+			 2
+			)*sizeof(char));
+	/* and form the query */
+	sprintf(connection->request, "%s %s \"%s\"",
+			connection->request,
+			mpdTagItemKeys[field],
+			arg);
+	free(arg);
+}
+
+
+void mpd_commitSearch(mpd_Connection *connection) 
+{
+	if(connection->request)
+	{
+		int length = strlen(connection->request);
+		/* fixing up the string for mpd to like */
+		connection->request = realloc(connection->request, 
+				(length+	/* old length */
+				 2		/* closing \n and \0 */
+				)*sizeof(char));
+		connection->request[length] = '\n';
+		connection->request[length+1] = '\0';
+		/* and off we go */
+		mpd_sendInfoCommand(connection, connection->request);
+		/* clean up a bit */
+		free(connection->request);
+		connection->request = NULL;
+	}
+}
+
